@@ -28,15 +28,17 @@
 #include <config.h>
 #include "nautilus-bookmarks-window.h"
 #include "nautilus-window.h"
-#include <libnautilus/nautilus-undo.h>
+#include "nautilus-navigation-window.h"
+#include "nautilus-spatial-window.h"
+#include <libnautilus-private/nautilus-undo.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gnome-extensions.h>
-#include <libegg/egg-screen-help.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-undo-signal-handlers.h>
 #include <gtk/gtkhbbox.h>
 #include <gnome.h>
+#include <libgnomeui/gnome-help.h>
 #include <glade/glade.h>
 
 
@@ -56,11 +58,16 @@ static int		     name_field_changed_signal_id;
 static GtkWidget	    *remove_button = NULL;
 static GtkWidget            *jump_button = NULL;
 static gboolean		     text_changed = FALSE;
+static gboolean		     name_text_changed = FALSE;
 static GtkWidget	    *uri_field = NULL;
 static int		     uri_field_changed_signal_id;
 static int		     row_changed_signal_id;
 static int		     row_deleted_signal_id;
 static int                   row_activated_signal_id;
+static int                   key_pressed_signal_id;
+static int                   jump_button_signal_id;
+static NautilusApplication  *application;
+static gboolean              parent_is_browser_window;
 
 /* forward declarations */
 static guint    get_selected_row                            (void);
@@ -86,6 +93,9 @@ static void	on_row_activated			    (GtkTreeView	  *view,
 							     GtkTreePath	  *path,
                                                              GtkTreeViewColumn    *column,
 							     gpointer		   user_data);
+static gboolean	on_key_pressed                              (GtkTreeView	  *view,
+                                                             GdkEventKey          *event,
+							     gpointer		   user_data);
 static void     on_selection_changed                        (GtkTreeSelection     *treeselection,
 							     gpointer              user_data);
 
@@ -103,6 +113,7 @@ static void     on_window_destroy_event                     (GtkWidget
 							     gpointer              user_data);
 static void     repopulate                                  (void);
 static void     set_up_close_accelerator                    (GtkWidget            *window);
+static void	open_selected_bookmark 			    (gpointer   user_data, GdkScreen *screen);
 
 /* We store a pointer to the bookmark in a column so when an item is moved
    with DnD we know which item it is. However we have to be careful to keep
@@ -133,8 +144,8 @@ nautilus_bookmarks_window_response_callback (GtkDialog *dialog,
 	if (response_id == GTK_RESPONSE_HELP) {
 		GError *error = NULL;
 
-		egg_help_display_desktop_on_screen (
-			NULL, "user-guide", "wgosnautilus.xml", "gosnautilus-36",
+		gnome_help_display_desktop_on_screen (
+			NULL, "user-guide", "user-guide.xml", "gosnautilus-36",
 			gtk_window_get_screen (GTK_WINDOW (dialog)), &error);
 
 		if (error) {
@@ -207,6 +218,22 @@ bookmarks_set_empty (gboolean empty)
 	on_selection_changed (bookmark_selection, NULL);
 }
 
+static void
+edit_bookmarks_dialog_reset_signals (gpointer data,
+				     GObject *obj)
+{
+	g_signal_handler_disconnect (GTK_OBJECT (jump_button),
+				     jump_button_signal_id);
+	g_signal_handler_disconnect (GTK_OBJECT (bookmark_list_widget),
+				     row_activated_signal_id);
+	jump_button_signal_id =
+		g_signal_connect (jump_button, "clicked",
+				  G_CALLBACK (on_jump_button_clicked), NULL);
+	row_activated_signal_id =
+		g_signal_connect (bookmark_list_widget, "row_activated",
+				  G_CALLBACK (on_row_activated), NULL);
+}
+
 /**
  * create_bookmarks_window:
  * 
@@ -236,14 +263,25 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
 		return NULL;
 	}
 
+	application = NAUTILUS_WINDOW (undo_manager_source)->application;
+
+	if (NAUTILUS_IS_NAVIGATION_WINDOW (undo_manager_source)) {
+		parent_is_browser_window = TRUE;
+	} else {
+		parent_is_browser_window = FALSE;
+	}
+
 	set_up_close_accelerator (window);
 	nautilus_undo_share_undo_manager (G_OBJECT (window), undo_manager_source);
 
 	gtk_window_set_wmclass (GTK_WINDOW (window), "bookmarks", "Nautilus");
 	nautilus_bookmarks_window_restore_geometry (window);
+
+	g_object_weak_ref (G_OBJECT (undo_manager_source), edit_bookmarks_dialog_reset_signals, 
+			   undo_manager_source);
 	
 	bookmark_list_widget = GTK_TREE_VIEW (glade_xml_get_widget (gui, "bookmark_tree_view"));
-		
+
 	rend = gtk_cell_renderer_pixbuf_new ();
 	col = gtk_tree_view_column_new_with_attributes ("Icon", 
 							rend,
@@ -275,6 +313,7 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
 		GTK_TREE_SELECTION (gtk_tree_view_get_selection (bookmark_list_widget));
 
 	name_field = nautilus_entry_new ();
+	
 	gtk_widget_show (name_field);
 	gtk_box_pack_start (GTK_BOX (glade_xml_get_widget (gui, "bookmark_name_placeholder")),
 			    name_field, TRUE, TRUE, 0);
@@ -306,6 +345,9 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
         row_activated_signal_id =
                 g_signal_connect (bookmark_list_widget, "row_activated",
                                   G_CALLBACK (on_row_activated), undo_manager_source);
+        key_pressed_signal_id =
+                g_signal_connect (bookmark_list_widget, "key_press_event",
+                                  G_CALLBACK (on_key_pressed), NULL);
 	selection_changed_id =
 		g_signal_connect (bookmark_selection, "changed",
 				  G_CALLBACK (on_selection_changed), NULL);	
@@ -338,8 +380,9 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
 			  G_CALLBACK (name_or_uri_field_activate), NULL);
 	g_signal_connect (remove_button, "clicked",
 			  G_CALLBACK (on_remove_button_clicked), NULL);
-	g_signal_connect (jump_button, "clicked",
-			  G_CALLBACK (on_jump_button_clicked), undo_manager_source);
+	jump_button_signal_id = 
+		g_signal_connect (jump_button, "clicked",
+				  G_CALLBACK (on_jump_button_clicked), undo_manager_source);
 
 	/* Register to find out about icon theme changes */
 	g_signal_connect_object (nautilus_icon_factory_get (), "icons_changed",
@@ -356,6 +399,26 @@ create_bookmarks_window (NautilusBookmarkList *list, GObject *undo_manager_sourc
 	return GTK_WINDOW (window);
 }
 
+void
+edit_bookmarks_dialog_set_signals (GObject *undo_manager_source)
+{
+
+	g_signal_handler_disconnect (GTK_OBJECT (jump_button), 
+				     jump_button_signal_id);
+	g_signal_handler_disconnect (GTK_OBJECT (bookmark_list_widget),
+				     row_activated_signal_id);
+
+	jump_button_signal_id =
+		g_signal_connect (jump_button, "clicked",
+				  G_CALLBACK (on_jump_button_clicked), undo_manager_source);
+	row_activated_signal_id =
+		g_signal_connect (bookmark_list_widget, "row_activated",
+				  G_CALLBACK (on_row_activated), undo_manager_source);
+
+	g_object_weak_ref (G_OBJECT (undo_manager_source), edit_bookmarks_dialog_reset_signals,
+			   undo_manager_source);
+}
+
 static NautilusBookmark *
 get_selected_bookmark (void)
 {
@@ -478,33 +541,66 @@ on_name_field_changed (GtkEditable *editable,
 			    gtk_entry_get_text (GTK_ENTRY (name_field)),
 			    -1);
 	text_changed = TRUE;
+	name_text_changed = TRUE;
 }
 
 static void
-go_to_selected_bookmark (NautilusWindow *window)
+open_selected_bookmark (gpointer user_data, GdkScreen *screen)
 {
-        NautilusBookmark *selected;
-        char *uri = NULL;
+	NautilusBookmark *selected;
+	NautilusWindow *window;
+	char *uri = NULL;
+	
+	selected = get_selected_bookmark ();
 
-        selected = get_selected_bookmark ();
+	if (!selected) {
+		return;
+	}
 
-        if (selected) {
-                uri = nautilus_bookmark_get_uri (selected);
-                nautilus_window_go_to (window, uri);
-                g_free (uri);
-        }
+	uri = nautilus_bookmark_get_uri (selected);
+
+	if (!uri) { 
+		return;
+	}
+
+	if (NAUTILUS_IS_NAVIGATION_WINDOW (user_data)) {
+		nautilus_window_go_to (NAUTILUS_WINDOW (user_data), uri);
+	} else if (NAUTILUS_IS_SPATIAL_WINDOW (user_data)) {
+		window = nautilus_application_present_spatial_window (application, 
+								      NULL,
+								      NULL,
+								      uri,
+								      screen);
+	} else { /* window that opened bookmarks window has been closed */
+		if (parent_is_browser_window || eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
+			window = nautilus_application_create_navigation_window (application, 
+										NULL,
+										screen);
+			nautilus_window_go_to (window, uri);
+		} else {
+			window = nautilus_application_present_spatial_window (application, 
+									      NULL,
+									      NULL,
+									      uri,
+									      screen);
+		}
+	}
+
+	g_free (uri);
 }
 
 static void
 on_jump_button_clicked (GtkButton *button,
                         gpointer   user_data)
 {
-        go_to_selected_bookmark (NAUTILUS_WINDOW (user_data));
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (GTK_WIDGET (button));
+	open_selected_bookmark (user_data, screen);
 }
 
 static void
-on_remove_button_clicked (GtkButton *button,
-                          gpointer   user_data)
+bookmarks_delete_bookmark (void)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
@@ -539,6 +635,14 @@ on_remove_button_clicked (GtkButton *button,
 	}
 }
 
+static void
+on_remove_button_clicked (GtkButton *button,
+                          gpointer   user_data)
+{
+        bookmarks_delete_bookmark ();
+}
+
+
 /* This is a bit of a kludge to get DnD to work. We check if the row in the
    GtkListStore matches the one in the bookmark list. If it doesn't, we assume
    the bookmark has just been dragged here and we insert it into the bookmark
@@ -588,13 +692,29 @@ on_row_changed (GtkListStore *store,
 	}
 }
 
+static gboolean
+on_key_pressed (GtkTreeView *view,
+                GdkEventKey *event,
+                gpointer user_data)
+{
+        if (event->keyval == GDK_Delete || event->keyval == GDK_KP_Delete) {
+                bookmarks_delete_bookmark ();
+                return TRUE;
+        }
+
+        return FALSE;
+}
+
 static void
 on_row_activated (GtkTreeView       *view,
                   GtkTreePath       *path,
                   GtkTreeViewColumn *column,
                   gpointer           user_data)
 {
-        go_to_selected_bookmark (NAUTILUS_WINDOW (user_data));
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (GTK_WIDGET (view));
+	open_selected_bookmark (user_data, screen);
 }
 
 static void
@@ -646,6 +766,7 @@ on_selection_changed (GtkTreeSelection *treeselection,
 	g_signal_handler_unblock (uri_field, uri_field_changed_signal_id);
 
 	text_changed = FALSE;
+	name_text_changed = FALSE;
 
 	g_free (name);
 	g_free (uri);
@@ -667,6 +788,8 @@ update_bookmark_from_text (void)
 			(gtk_entry_get_text (GTK_ENTRY (uri_field)),
 			 gtk_entry_get_text (GTK_ENTRY (name_field)));
 
+		nautilus_bookmark_set_has_custom_name (bookmark, name_text_changed);
+
 		selected_row = get_selected_row ();
 
 		/* turn off list updating 'cuz otherwise the list-reordering code runs
@@ -785,11 +908,15 @@ repopulate (void)
 				row_deleted_signal_id);
         g_signal_handler_block (bookmark_list_widget,
                                 row_activated_signal_id);
+        g_signal_handler_block (bookmark_list_widget,
+                                key_pressed_signal_id);
 
 	gtk_list_store_clear (store);
 	
 	g_signal_handler_unblock (bookmark_list_widget,
 				  row_activated_signal_id);
+        g_signal_handler_unblock (bookmark_list_widget,
+                                  key_pressed_signal_id);
 	g_signal_handler_unblock (bookmark_list_store,
 				  row_deleted_signal_id);
 	g_signal_handler_unblock (bookmark_selection,
